import {
  Action,
  getModule,
  Module,
  Mutation,
  MutationAction,
  VuexModule
} from 'vuex-module-decorators';
import { LoginSubmit } from '@/models';
import store from '@/store';
import api from '@/services/pm-api-client';
import Cookies from 'js-cookie';

const COOKIE_TOKEN = 'pm-api-cookie-token';

@Module({ namespaced: true, name: 'auth', store, dynamic: true })
class AuthStoreModule extends VuexModule {
  public authToken: string | undefined = undefined;

  @Action({ commit: 'SET_TOKEN', rawError: true })
  public async login(creds: LoginSubmit) {
    return await api.createAuthToken(creds);
  }

  @Action({ commit: 'SET_TOKEN', rawError: true })
  public async findLocalToken() {
    const token = Cookies.get(COOKIE_TOKEN);
    if (token) { return token; }
    else { throw new Error('No auth token stored as a cookie.'); }
  }

  @Action({ commit: 'SET_TOKEN' })
  public setToken(t: string) {
    // TODO: set the expires based on the JWT token expiry?
    Cookies.set(COOKIE_TOKEN, t, { secure: true });
    return t;
  }

  @Mutation public SET_TOKEN(t: string) {
    this.authToken = t;
  }
}

export const authStore = getModule(AuthStoreModule);
export default authStore;
